#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" Author:                     Hendrik Schutter, mail@hendrikschutter.com
    Date of creation:           2022/03/01
    Date of last modification:  2023/03/02
"""

from http.server import BaseHTTPRequestHandler, HTTPServer
import paho.mqtt.client as mqtt
from datetime import datetime
import threading
import time
import json
import config

scrape_healthy = True
startTime = datetime.now()
node_metrics = list()
mutex = threading.Lock()
request_count = 0

class RequestHandler(BaseHTTPRequestHandler):

    def get_metrics(self):
        global request_count
        global node_metrics
        global mutex
        mutex.acquire()
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()
        self.wfile.write(bytes(config.exporter_prefix + "expoter_duration_seconds_sum " + str(int((datetime.now() - startTime).total_seconds())) + "\n", "utf-8"))
        self.wfile.write(bytes(config.exporter_prefix + "exporter_request_count " + str(request_count) + "\n", "utf-8"))
        self.wfile.write(bytes(config.exporter_prefix + "exporter_scrape_healthy " + str(int(scrape_healthy)) + "\n", "utf-8"))
        
        for metric in node_metrics:
            #print(metric)
            self.wfile.write(bytes(config.exporter_prefix + metric + "\n", "utf-8"))

        mutex.release()
        
    def do_GET(self):
        global request_count
        request_count = request_count + 1
        print("Request: " + self.path)
        if (self.path.startswith("/metrics")): 
            self.get_metrics()
        else:
            self.send_response(200)
            self.send_header("Content-type", "text/html")
            self.end_headers()
            self.wfile.write(bytes("<html>", "utf-8"))
            self.wfile.write(bytes("<head><title>MSV Clubhouse exporter</title></head>", "utf-8"))
            self.wfile.write(bytes("<body>", "utf-8"))
            self.wfile.write(bytes('<h1>msv-clubhouse exporter based on data from LoRaWAN TTN node.</h1>', "utf-8"))
            self.wfile.write(bytes('<p><a href="/metrics">Metrics</a></p>', "utf-8"))
            self.wfile.write(bytes("</body>", "utf-8"))
            self.wfile.write(bytes("</html>", "utf-8"))

def update_metrics(payload, metadata):
    print("Payload: "+ str(payload))
    print("Metadata: "+ str(metadata))

    print("set data from ttn")
    global node_metrics
    global mutex
    global scrape_healthy
    mutex.acquire()
    scrape_healthy = True
    node_metrics.clear()

    try:
            node_metrics.append("target_temperature " + str(int(42)))
            node_metrics.append("valve " + str(int(42)))
            node_metrics.append("low_battery " + str(int(42)))
            node_metrics.append("window_open " + str(int(42)))
    except Exception as ex:
        print("unable to poll data from TTN! error: " + str(ex))
        scrape_healthy = False
        pass
    mutex.release()
    time.sleep(300)

def on_connect(mqttc, obj, flags, rc):
    print("\nConnected to MQTT: rc = " + str(rc))

def on_message(mqttc, obj, msg):
    #print("\nMessage: " + msg.topic + " " + str(msg.qos))
    parsedJSON = json.loads(msg.payload)
    #print(json.dumps(parsedJSON, indent=4))
    uplink_message = parsedJSON["uplink_message"];
    update_metrics(uplink_message["decoded_payload"], uplink_message["rx_metadata"])

def on_subscribe(mqttc, obj, mid, granted_qos):
    print("\nSubscribed to MQTT: " + str(mid) + " " + str(granted_qos))

def poll_mqtt(mqttc):
	while True:
			mqttc.loop(10) 	# seconds timeout

def main():
	print("starting ...")	

	mqttc = mqtt.Client()
	mqttc.on_connect = on_connect
	mqttc.on_subscribe = on_subscribe
	mqttc.on_message = on_message
	mqttc.username_pw_set(config.ttn_user, config.ttn_key)
	mqttc.tls_set()	
	mqttc.connect(config.ttn_region.lower() + ".cloud.thethings.network", 8883, 60)
	mqttc.subscribe("#", 0)	# all device uplinks

    # run mqtt in thread forever
	poll_mqtt_thread = threading.Thread(target=poll_mqtt, args=((mqttc,)))
	poll_mqtt_thread.start()

	webServer = HTTPServer((config.hostName, config.serverPort), RequestHandler)
	print("Server started http://%s:%s" % (config.hostName, config.serverPort))
	try:
		webServer.serve_forever()
	except KeyboardInterrupt:
		pass

	webServer.server_close()
	print("Server stopped.")
	poll_mqtt_thread.join()

if __name__ == "__main__":
	main()